import openai
import time
import os

# from tenacity import (
#     retry,
#     stop_after_attempt, # type: ignore
#     wait_random_exponential, # type: ignore
# )

from llamp.llms.base_llm_system import BaseLLMSystem

class OpenAIChatText(BaseLLMSystem):
    def __init__(self, system_name="OpenAIChatText",save_path="game_logs", temperature=0.0, model="gpt-3.5-turbo-0125", stop_sequences=None):
        
        super().__init__(system_name, save_path, temperature=temperature)        
        self.client = openai.OpenAI(
        # Defaults to os.environ.get("OPENAI_API_KEY")
        # api_key=OPENAI_KEY,
        )
        self.openai_attempts = 0

        self.model = model
        self.stop_sequences = stop_sequences


    # @retry(wait=wait_random_exponential(min=1, max=60), stop=stop_after_attempt(6), reraise=True)
    def call_model(self, temperature=None):
        """Call OpenAI API"""

        prompt = self.generate_text_prompt()

        full_prompt = [{
                "role": "user", 
                "content": prompt
        }]

        chat_completion = self.client.chat.completions.create(
            model=self.model,
            messages=full_prompt,
            temperature=self.temperature,
            stop = self.stop_sequences

        )
        chat_message = chat_completion.choices[0].message.content

        return chat_message


if __name__=="__main__":
    print("Nothing to run here.")
    agent = OpenAIChatText(save_path="./")
    prompt = []
    agent.save()

    agent.set_base_prompt_and_reset(prompt)


    observation = "Hi"
    action = agent.act(observation)

    print(action)

    agent.save()

    # EOF


# import openai
import time
import os

from cerebras.cloud.sdk import Cerebras



# from tenacity import (
#     retry,
#     stop_after_attempt, # type: ignore
#     wait_random_exponential, # type: ignore
# )

from llamp.llms.base_llm_system import BaseLLMSystem

class CerebrasChatText(BaseLLMSystem):
    def __init__(self, system_name="CerebrasChatText",save_path="game_logs", temperature=0.0, model="llama3.1-8b", stop_sequences=None):
        
        super().__init__(system_name, save_path, temperature=temperature)    
        self.client  = Cerebras(
            # This is the default and can be omitted
            api_key=os.environ.get("CEREBRAS_API_KEY"),
        )    
        # self.client = openai.OpenAI(
        #   base_url = "https://api.cerebras.ai/v1/chat/completions",
        #   api_key = os.environ.get("CEREBRAS_API_KEY")
        # )
        self.openai_attempts = 0

        self.model = model
        self.stop_sequences = stop_sequences


    # @retry(wait=wait_random_exponential(min=1, max=60), stop=stop_after_attempt(6), reraise=True)
    def call_model(self, temperature=None):
        """Call CEREBRAS API"""

        prompt = self.generate_text_prompt()

        full_prompt = [{
                "role": "user", 
                "content": prompt
        }]

        chat_completion = self.client.chat.completions.create(
            model=self.model,
            messages=full_prompt,
            temperature=self.temperature,
            stop = self.stop_sequences

        )
        chat_message = chat_completion.choices[0].message.content

        return chat_message


if __name__=="__main__":
    print("Nothing to run here.")
    agent = CerebrasChatText(save_path="./")
    prompt = []
    agent.save()

    agent.set_base_prompt_and_reset(prompt)


    observation = "Hi"
    action = agent.act(observation)

    print(action)

    agent.save()

    # EOF


import openai
import time
import os

# from tenacity import (
#     retry,
#     stop_after_attempt, # type: ignore
#     wait_random_exponential, # type: ignore
# )

from llamp.llms.base_llm_system import BaseLLMSystem

class OpenAIText(BaseLLMSystem):
    def __init__(self, system_name="OpenAIText",save_path="game_logs", temperature=0.0, model="davinci-002", stop_sequences=None):
        
        super().__init__(system_name, save_path, temperature=temperature)        
        self.client = openai.OpenAI(
        # Defaults to os.environ.get("OPENAI_API_KEY")
        # api_key=OPENAI_KEY,
        )
        self.openai_attempts = 0

        self.model = model
        self.stop_sequences = stop_sequences


    # @retry(wait=wait_random_exponential(min=1, max=60), stop=stop_after_attempt(6), reraise=True)
    def call_model(self, temperature=None):
        """Call OpenAI API"""

        prompt = self.generate_text_prompt()
        # print("=+"*20)
        # print(prompt)
        # input(">")

        completion = self.client.completions.create(
            model=self.model,
            prompt=prompt,
            temperature=self.temperature,
            stop=self.stop_sequences,
            max_tokens=250
        )
        message = completion.choices[0].text
        # print("-+"*20)
        # print(message)
        # print(completion.usage)
        # print("-+"*20)

        return message




if __name__=="__main__":
    print("Nothing to run here.")
    # def call_model(self, attempt_limit = 100):
    #   """Call OpenAI API"""
    #   try:
    #       print("Trying to call GPT")
    #       chat_completion = self.client.chat.completions.create(
    #           # model="gpt-3.5-turbo",
    #           model="gpt-3.5-turbo-0125",
    #           # model="gpt-4-turbo-preview",
    #           messages=self.current_prompt,
    #           temperature=0.8
    #       )
    #       chat_message = chat_completion.choices[0].message.content
    #       self.openai_attempts = 0
    #   except openai.RateLimitError:
    #       print(f"Need to wait ... attempt:{self.openai_attempts}")
    #       time.sleep(20)
    #       self.openai_attempts+=1
    #       if self.openai_attempts < attempt_limit:
    #           return self.call_model()
    #       else:
    #           raise Exception("Exceeded Attempt Limit")

    #   return chat_message


from llamp.llms.local.minichat import MiniChat

if __name__=="__main__":
    agent = MiniChat(test_mode=True)
    # agent.add_first_observation("Yes?")

    agent.act("yes?")

    prompt = agent.construct_prompt_for_model()
    print(prompt)


import time
import os
import cohere

from llamp.llms.base_llm_system import BaseLLMSystem

# from tenacity import (
#     retry,
#     stop_after_attempt, # type: ignore
#     wait_random_exponential, # type: ignore
# )

class CohereChat(BaseLLMSystem):
    def __init__(self, system_name="CohereChat",save_path="game_logs", temperature = 0.0, model="command", stop_sequences=None):
        
        super().__init__(system_name, save_path, temperature=temperature)        
        self.base_prompt = [{
            "role" : "system",
            "content" : "You will interact with the environment to solve the given task. Think step by step "
        }]
        self.reset()
        API_KEY = os.environ['COHERE_API_KEY']
        self.co = cohere.Client(API_KEY)

        self.temperature = temperature
        self.model = model
        self.stop_sequences=stop_sequences

    
    def get_current_prompt_cohere(self):
        """Returns the current prompt and last message for Cohere's API"""
        out_list = []
        for messages in self.current_prompt:
            temp_dict = {}
            if messages["role"] == "assistant":
                cohere_role = "CHATBOT" 
            elif messages["role"] == "user":
                cohere_role = "USER"
            elif messages["role"] == "system":
                cohere_role = "SYSTEM"
            else:
                cohere_role = "user"
            temp_dict["role"] = cohere_role
            temp_dict["message"] = messages["content"]
            out_list.append(temp_dict)

        return out_list

    # @retry(wait=wait_random_exponential(min=1, max=60), stop=stop_after_attempt(6),reraise=True)
    def call_model(self, temperature=None):
        """Call OpenAI API"""

        current_prompt = self.get_current_prompt_cohere()

        message = current_prompt[-1]["message"]
        if len(current_prompt)>1:
            chat_history = current_prompt[:-1]

        response = self.co.chat(
            message = message,
            model=self.model,
            chat_history=chat_history,
            temperature=self.temperature,
            stop_sequences=self.stop_sequences
        )

        answer = response.text
        return answer


if __name__=="__main__":
    print("Nothing to run here.")



 


import time
import os

import anthropic
from anthropic import HUMAN_PROMPT, AI_PROMPT

from llamp.llms.base_llm_system import BaseLLMSystem


# from tenacity import (
#     retry,
#     stop_after_attempt, # type: ignore
#     wait_random_exponential, # type: ignore
# )


class AnthropicText(BaseLLMSystem):
    def __init__(self, system_name="AnthropicText",save_path="game_logs", temperature=0.0, model="claude-2.1", stop_sequences=None):
        
        super().__init__(system_name, save_path, temperature=temperature)        
        self.client = anthropic.Anthropic(
            # defaults to os.environ.get("ANTHROPIC_API_KEY")
            # api_key="my_api_key",
        )
        # self.openai_attempts = 0

        self.temperature = temperature
        self.model = model
        self.stop_sequences = stop_sequences

    
    # @retry(wait=wait_random_exponential(min=1, max=60), stop=stop_after_attempt(6), reraise=True)
    def call_model(self):
        """Call OpenAI API"""
        prompt = self.generate_text_prompt()

        completion = self.client.completions.create(
            model="claude-2.1",
            max_tokens_to_sample=1024,
            prompt=f"{HUMAN_PROMPT} {prompt}{AI_PROMPT}",
            stop_sequences=self.stop_sequences
        )
        
        response = completion.completion[0]

        return response




if __name__=="__main__":
    print("Nothing to run here.")
    # def call_model(self, attempt_limit = 100):
    #   """Call OpenAI API"""
    #   try:
    #       print("Trying to call GPT")
    #       chat_completion = self.client.chat.completions.create(
    #           # model="gpt-3.5-turbo",
    #           model="gpt-3.5-turbo-0125",
    #           # model="gpt-4-turbo-preview",
    #           messages=self.current_prompt,
    #           temperature=0.8
    #       )
    #       chat_message = chat_completion.choices[0].message.content
    #       self.openai_attempts = 0
    #   except openai.RateLimitError:
    #       print(f"Need to wait ... attempt:{self.openai_attempts}")
    #       time.sleep(20)
    #       self.openai_attempts+=1
    #       if self.openai_attempts < attempt_limit:
    #           return self.call_model()
    #       else:
    #           raise Exception("Exceeded Attempt Limit")

    #   return chat_message


import time
import os
import cohere

from llamp.llms.base_llm_system import BaseLLMSystem

# from tenacity import (
#     retry,
#     stop_after_attempt, # type: ignore
#     wait_random_exponential, # type: ignore
# )

class CohereChatText(BaseLLMSystem):
    def __init__(self, system_name="CohereChatText",save_path="game_logs", temperature = 0.0, model="command", stop_sequences=None):
        
        super().__init__(system_name, save_path, temperature=temperature)        
        self.base_prompt = [{
            "role" : "system",
            "content" : "You will interact with the environment to solve the given task. Think step by step "
        }]
        self.reset()
        API_KEY = os.environ['COHERE_API_KEY']
        self.co = cohere.Client(API_KEY)

        self.temperature = temperature
        self.model = model
        self.stop_sequences = stop_sequences

    # @retry(wait=wait_random_exponential(min=1, max=60), stop=stop_after_attempt(6),reraise=True)
    def call_model(self, temperature=None):
        """Call OpenAI API"""
        message = self.generate_text_prompt()

        response = self.co.chat(
            message = message,
            model=self.model,
            temperature=self.temperature,
            prompt_truncation = "AUTO_PRESERVE_ORDER",
            stop_sequences=self.stop_sequences
        )

        answer = response.text
        return answer


    # def post_process_model_output(self, model_output):
    #     """Manual shortening (as Cohere API doesn't allow for it yet)"""
    #     truncated_model_output = model_output.split(self.stop_sequences[0])[0]
    #     # Simple truncation for now.
    #     return truncated_model_output



if __name__=="__main__":
    print("Nothing to run here.")



 


from transformers import AutoTokenizer
from vllm import LLM, SamplingParams


class QwenLLM():

    def __init__(self, model_name="Qwen/Qwen2.5-7B-Instruct", tensor_parallel_size=1):
        # Initialize the tokenizer
        self.tokenizer = AutoTokenizer.from_pretrained(model_name)

        # Pass the default decoding hyperparameters of Qwen2.5-7B-Instruct
        # max_tokens is for the maximum length for generation.
        self.sampling_params = SamplingParams(temperature=0.7, top_p=0.8, repetition_penalty=1.05, max_tokens=512)

        # Input the model name or path. Can be GPTQ or AWQ models.
        self.llm = LLM(model=model_name, tensor_parallel_size=tensor_parallel_size)

    def call_llm(self,prompt):
        """Calls the llm and does stuff"""
        # Prepare your prompts
        messages = [
            {"role": "system", "content": "You are Qwen, created by Alibaba Cloud. You are a helpful assistant."},
            {"role": "user", "content": prompt}
        ]
        text = self.tokenizer.apply_chat_template(
            messages,
            tokenize=False,
            add_generation_prompt=True
        )

        # generate outputs
        outputs = self.llm.generate([text], self.sampling_params)

        # Print the outputs.
        for output in outputs:
            prompt = output.prompt
            generated_text = output.outputs[0].text
            print(f"Prompt: {prompt!r}, Generated text: {generated_text!r}")


if __name__ ==  "__main__":
    prompt = "Tell me something about large language models."
    llm = QwenLLM()
    llm.call_llm(prompt)



from llamp.llms.base_llm_system import BaseLLMSystem
from transformers import AutoTokenizer
from vllm import LLM, SamplingParams


class VLLMChat(BaseLLMSystem):
    def __init__(self, system_name="VLLMChat", save_path="game_logs", temperature=0.0, model="Qwen/Qwen2.5-7B-Instruct", tensor_parallel_size=1, max_model_len=16000, quantization=False):
        super().__init__(system_name, save_path, temperature=temperature)
        
        self.tokenizer = AutoTokenizer.from_pretrained(model)
        self.max_model_len = max_model_len

        try:
            self.apply_chat_template("test")
            self.chat_model = True
        except:
            self.chat_model = False
            
        self.sampling_params = SamplingParams(
            temperature=temperature,
            top_p=1.0,
            repetition_penalty=1.00,
            max_tokens=2000
        )
        if not quantization:
            self.llm = LLM(
                model=model,
                tensor_parallel_size=tensor_parallel_size,
                gpu_memory_utilization=0.95,
                max_model_len=max_model_len,
                dtype="auto"
            )
        else:
            self.llm = LLM(
                model=model,
                tensor_parallel_size=tensor_parallel_size,
                gpu_memory_utilization=0.95,
                max_model_len=max_model_len,
                dtype="auto",
                quantization="bitsandbytes", 
                load_format="bitsandbytes"
            )
        print("="*20)
        print(f"Model loaded as: {model} with tensors: {tensor_parallel_size}")

    def apply_chat_template(self,prompt):
        """uses the chat template"""
        messages = [
            # {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": prompt[-self.max_model_len:]}
        ]
        text = self.tokenizer.apply_chat_template(
            messages,
            tokenize=False,
            add_generation_prompt=True
        )
        return text

    def call_model(self, temperature=None):
        prompt = self.generate_text_prompt()

        if self.chat_model:
            text = self.apply_chat_template(prompt)
        else:
            text = prompt

        outputs = self.llm.generate([text], self.sampling_params)
        return outputs[0].outputs[0].text

    def count_tokens(self, optional_text=None, model_name=None):
        if optional_text:
            return len(self.tokenizer.encode(optional_text))
        else:
            prompt = self.generate_text_prompt()
            return len(self.tokenizer.encode(prompt))


if __name__=="__main__":
    import argparse
    
    parser = argparse.ArgumentParser(description='Run QwenLLM System')
    parser.add_argument('--model', type=str, default="Qwen/Qwen2.5-7B-Instruct",
                        # choices=["Qwen/Qwen2.5-7B-Instruct", 
                        #         "Qwen/Qwen2.5-14B-Instruct",
                        #         "Qwen/Qwen2.5-32B-Instruct",
                        #         "mistralai/Mixtral-8x7B-Instruct-v0.1",
                        #         "mistralai/Mixtral-8x22B-Instruct-v0.1",
                        #         ],
                        help='Model to use')
    parser.add_argument('--gpus', type=int, default=1,
                        help='Number of GPUs to use for tensor parallelism')
    parser.add_argument('--max_len', type=int, default=5000,
                        help='Num_Max_len')
    parser.add_argument('--temperature', type=float, default=0.7,
                        help='Temperature for sampling')
    parser.add_argument('--test_prompt', type=str, 
                        default="Tell me a short story about a robot learning to paint.",
                        help='Test prompt to try')
    parser.add_argument("--quantization", type=int, default=0, help="Whether a quantized model is being loaded.")
    # parser.add_argument('--force_model', type=str, 
    #                     default="Tell me a short story about a robot learning to paint.",
    #                     help='Test prompt to try')    
    args = parser.parse_args()
    
    system = VLLMChat(
        model=args.model,
        tensor_parallel_size=args.gpus,
        temperature=args.temperature,
        max_model_len=args.max_len,
        quantization=bool(args.quantization)
    )
    
    print("\nTesting the act method:")
    response, token_info = system.act(args.test_prompt, return_token_count=True)
    print(f"\nResponse:\n{response}")
    print(f"\nToken Information:")
    print(f"Input Tokens: {token_info['in_token_all']}")
    print(f"Message Tokens: {token_info['in_token_message']}")
    print(f"Output Tokens: {token_info['out_token_action']}")
    system.save()

    # print(f"\nTesting with prompt: {args.test_prompt}")
    # system.add_message("user", args.test_prompt)
    # response = system.call_model()
    # print(f"\nResponse:\n{response}")
    
    # token_count = system.count_tokens()
    # print(f"\nToken count: {token_count}")



from llamp.llms.base_system import BaseSystem

class Human(BaseSystem):
    def __init__(self, system_name="Human",save_path="game_logs"):
        super().__init__(system_name, save_path)
   

    def call_model(self):
        """The action is just whatever the user inputs"""
        action = input("[[INPUT YOUR ACTION]] >>")
        return action     


if __name__=="__main__":
    print("Nothing to run here.")



import openai
import time
import os

# from tenacity import (
#     retry,
#     stop_after_attempt, # type: ignore
#     wait_random_exponential, # type: ignore
# )

from llamp.llms.base_llm_system import BaseLLMSystem

class OpenAIChatTextSampling(BaseLLMSystem):
    def __init__(self, system_name="OpenAIChatTextSampling",save_path="game_logs", temperature=0.0, model="gpt-3.5-turbo-0125", stop_sequences=None, resample_temperature_jump=0.1):
        
        super().__init__(system_name, save_path, temperature=temperature)        
        self.client = openai.OpenAI(
        # Defaults to os.environ.get("OPENAI_API_KEY")
        # api_key=OPENAI_KEY,
        )
        self.openai_attempts = 0
        self.model = model

        self.original_temperature = temperature

        self.current_sample = 0
        self.resample_temperature_jump = resample_temperature_jump

        self.previous_samples = {}
        self.resampling = False

        self.stop_sequences = stop_sequences
        self.__PREVIOUS_SAMPLES_KEY = "previous_samples"
        self.__DEFAULT_HISTORY_KEYS = ["role","content","temperature","out_token_action"]

    def act(self, current_observation="", temperature=None, return_token_count=False):
        """ Acting"""
        if not self.resampling:
            if not current_observation:
                raise Exception("You have to give an observation when you are not resampling.")
            self.temperature = self.original_temperature
            self.previous_samples = {}
            additional_logging_in = self._count_in_token(current_observation)
            self.add_to_history(current_observation, "user", additional_logging_in)

            in_tokens_count = self.count_tokens()
            assert additional_logging_in["in_token_all"]==in_tokens_count, "In Tokens need to be same length."


        action = self.call_model(temperature)
        additional_logging_out = self._count_out_token(action)

        action = self.post_process_model_output(action)

        if self.previous_samples:
            additional_logging = {}
            additional_logging.update(additional_logging_out)
            additional_logging.update(self.previous_samples)
        else:
            additional_logging = additional_logging_out

        self.add_to_history(action, "assistant", additional_data=additional_logging)

        self.resampling = False

        additional_logging_out.update(
            {
                "in_token_all" : self.full_history[-2]["in_token_all"],
                "in_token_message" : self.full_history[-2]["in_token_message"],
            }
        )
        if return_token_count:
            return action, additional_logging_out
        else:
            return action

    def prepare_resample(self, increase_temperature=True):
        """
        Stores the last interaction in another place and prepares for resample, including tracking of temperature.)
            
        Memory Structure:
        'user':,
        'content':,
        'temperature':,
        'previous_samples':[
            '0.0' : {'user':,'content':,'temperature':},
        ]
        """
        self.resampling = True
        previous_memory=self.pop_from_history(return_full=True)
        
        if not previous_memory:
            self.previous_samples = {}
            return

        else:
            previous_turn = {}
            for key in self.__DEFAULT_HISTORY_KEYS:
                previous_turn[key] = previous_memory[key]

            previous_samples = {}
            previous_samples_list = previous_memory.get(self.__PREVIOUS_SAMPLES_KEY)
            if not previous_samples_list:
                previous_samples_list = []
                

            previous_samples_list.append(previous_turn)
            previous_samples[self.__PREVIOUS_SAMPLES_KEY] = previous_samples_list

            self.previous_samples = previous_samples

        if increase_temperature:
            self.temperature += self.resample_temperature_jump


    def resample(self, increase_temperature=True):
        """ Samples again with increased temperature. """
        self.prepare_resample(increase_temperature)
        action = self.act()
        return action

    # @retry(wait=wait_random_exponential(min=1, max=60), stop=stop_after_attempt(6), reraise=True)
    def call_model(self, temperature=None):
        """Call OpenAI API"""
        prompt = self.generate_text_prompt()

        full_prompt = [{
                "role": "user", 
                "content": prompt
        }]
        chat_completion = self.client.chat.completions.create(
            model=self.model,
            messages=full_prompt,
            temperature=self.temperature,
            stop = self.stop_sequences

        )
        chat_message = chat_completion.choices[0].message.content

        return chat_message

    def is_resample(self):
        """Returns whether this system is designed for resampling."""
        return True


if __name__=="__main__":
    agent = OpenAIChatTextSampling(save_path="./")
    prompt = []
    agent.set_base_prompt_and_reset(prompt)

    observation = "Hi"
    print("----")
    print("0")
    action = agent.act(observation)

    print(action)

    agent.save()
    agent.load_from_saved_data(agent.file_name)
    print("----")
    print("1")
    action = agent.act("Tell me a joke")
    print(action)
    agent.prepare_resample()
    print("----")
    print("2")
    action = agent.act("Tell me a joke2") #it will ignore the new utterance
    print(action)
    agent.prepare_resample(increase_temperature=False) 
    print("----")
    print("3")
    action = agent.act("") #utterance can be empty
    
    agent.prepare_resample() 
    print("----")
    print("4")
    action = agent.act("") #utterance can be empty

    print("----")
    print("5")
    action = agent.act("Something about airplanes would be funnier.") #utterance can be empty?
    print(action)

    agent.save()
    agent2 = OpenAIChatTextSampling(save_path="./",resample_temperature_jump=0.3)
    agent2.load_from_saved_data(agent.file_name)
    print("----")
    print("6")
    action = agent2.resample()
    print(action)
    agent2.save()


